/*  guiftp - X window ftp client
 *
 *  Copyright (C) 1999  Luc Dufresne - ldufresne@altern.org
 *                      26, rue des Comices
 *                      59650 Villeneuve d'Ascq
 *                      FRANCE
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  See the file COPYING
 */

#include <gtk/gtk.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "options.h"
#include "protos.h"

GtkWidget *txttimeout;
GtkWidget *labelt;
GtkWidget *button1,*button2,*button3;

static void get_txt(GString **dest, GtkWidget *txt)
{
	gchar *temp;
	
	temp = gtk_entry_get_text( GTK_ENTRY(txt));
	*dest=g_string_new("");
	g_string_sprintf(*dest,"%s",temp);
}

static void save_options()
{
	GString *buf;
	FILE *fichier;
	
	buf=g_string_new("");
	g_string_sprintf(buf,"%s/."PACKAGE"/options",getenv("HOME"));
	if(NULL!=(fichier=fopen(buf->str,"wt")))
	{
		fprintf(fichier,"# Automatically generated file.\n# Do not edit (or make a backup copy before doing so)\n");
		fprintf(fichier,"toolbar:%d\n",options.ShowToolbar);
		fprintf(fichier,"toolbartype:%d\n",options.ToolBarType);
		fprintf(fichier,"hiddenfiles:%d\n",options.ShowHiddenFiles);
		fprintf(fichier,"sortdirectories:%d\n",options.SortDirectoriesFirst);
		fprintf(fichier,"timeout:%d\n",options.Timeout);
		fclose(fichier);
	
	}
	g_string_free(buf,TRUE);;
}

static void change_options(GtkWidget *w, gpointer data)
{
	GString *timeoutstr;
	get_txt(&timeoutstr,txttimeout);
	gtk_widget_destroy(w);
	options.Timeout=atoi(timeoutstr->str);
	if(options.Timeout<0)options.Timeout=-options.Timeout;
	g_string_free(timeoutstr,TRUE);
	if(options.ShowToolbar)	change_toolbar_style();
	else hide_toolbar(TRUE);
	save_options();
}

static void sortdirectories_callback (GtkWidget *widget, gpointer data)
{
	if (GTK_TOGGLE_BUTTON (widget)->active)
	{
        	options.SortDirectoriesFirst=1;
	}
	else{
		options.SortDirectoriesFirst=0;
        }
}

static void hiddenfile_callback (GtkWidget *widget, gpointer data)
{
	if (GTK_TOGGLE_BUTTON (widget)->active)
	{
        	options.ShowHiddenFiles=1;
	}
	else{
		options.ShowHiddenFiles=0;	
        }
}

static void showtoolbar_callback (GtkWidget *widget, gpointer data)
{
	if (GTK_TOGGLE_BUTTON (widget)->active)
	{
        	options.ShowToolbar=1;
		gtk_widget_set_sensitive(labelt,1);
		gtk_widget_set_sensitive(button1,1);
		gtk_widget_set_sensitive(button2,1);
		gtk_widget_set_sensitive(button3,1);
	}
	else{
		options.ShowToolbar=0;
		gtk_widget_set_sensitive(labelt,0);
		gtk_widget_set_sensitive(button1,0);
		gtk_widget_set_sensitive(button2,0);
		gtk_widget_set_sensitive(button3,0);
        }
}

static void toolbartext_callback (GtkWidget *widget, gpointer data)
{
	options.ToolBarType=1;
}

static void toolbarpics_callback (GtkWidget *widget, gpointer data)
{
	options.ToolBarType=2;
}

static void toolbarfull_callback (GtkWidget *widget, gpointer data)
{
	options.ToolBarType=0;
}

void edit_options()
{
	GtkWidget *window;
	GtkWidget *vbox, *vbox2;
	GtkWidget *hbox;
	GtkWidget *bbox;
	GtkWidget *button;
	GtkWidget *label;
	GtkWidget *frame;
	GString *buffer;
	
	if(working)return;
	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_window_set_modal (GTK_WINDOW(window),TRUE);
	gtk_signal_connect (GTK_OBJECT (window), "delete_event", gtk_widget_destroy, NULL);
	gtk_signal_connect (GTK_OBJECT (window), "destroy",
			  GTK_SIGNAL_FUNC(gtk_main_quit),
			  &window);
	gtk_window_set_title (GTK_WINDOW (window), _("Options"));
	gtk_container_set_border_width (GTK_CONTAINER (window), 5);
	gtk_widget_set_usize (window, 350, 220);
	gtk_window_set_position(GTK_WINDOW(window),GTK_WIN_POS_MOUSE);
	vbox = gtk_vbox_new (FALSE, 0);
	gtk_container_add (GTK_CONTAINER (window), vbox);

	frame=gtk_frame_new(_("Toolbar"));
	gtk_container_add (GTK_CONTAINER (vbox), frame);
	vbox2 = gtk_vbox_new (FALSE, 0);
	gtk_container_add (GTK_CONTAINER (frame), vbox2);
	
	button=gtk_check_button_new_with_label(_("Show Toolbar"));
	gtk_box_pack_start (GTK_BOX (vbox2),button, TRUE, TRUE, 0);

	if(options.ShowToolbar)gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (button), TRUE);

	labelt = gtk_label_new (_("Show Toolbar As"));
	gtk_misc_set_alignment (GTK_MISC (labelt), 0, 0);
	gtk_box_pack_start (GTK_BOX (vbox2), labelt, FALSE, FALSE, 0);
	hbox = gtk_hbox_new (FALSE, 0);
	gtk_box_pack_start (GTK_BOX (vbox2), hbox, TRUE, TRUE, 0);
	button1 = gtk_radio_button_new_with_label (NULL, _("Pictures and Text"));
	gtk_signal_connect (GTK_OBJECT (button1), "clicked",GTK_SIGNAL_FUNC (toolbarfull_callback),NULL);
	gtk_box_pack_start (GTK_BOX (hbox), button1, TRUE, TRUE, 0);
	if(options.ToolBarType==0)gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button1), TRUE);
	button2 = gtk_radio_button_new_with_label(gtk_radio_button_group (GTK_RADIO_BUTTON (button1)), _("Pictures Only"));
	gtk_signal_connect (GTK_OBJECT (button2), "clicked",GTK_SIGNAL_FUNC (toolbarpics_callback),NULL);
	gtk_box_pack_start (GTK_BOX (hbox), button2, TRUE, TRUE, 0);
	if(options.ToolBarType==2)gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button2), TRUE);
	button3 = gtk_radio_button_new_with_label(gtk_radio_button_group (GTK_RADIO_BUTTON (button2)), _("Text Only"));
	gtk_signal_connect (GTK_OBJECT (button3), "clicked",GTK_SIGNAL_FUNC (toolbartext_callback),NULL);
	gtk_box_pack_start (GTK_BOX (hbox), button3, TRUE, TRUE, 0);
	if(options.ToolBarType==1)gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button3), TRUE);

	gtk_signal_connect_object (GTK_OBJECT (button), "toggled",
		showtoolbar_callback, GTK_OBJECT(button));

	if(!options.ShowToolbar)
	{
		gtk_widget_set_sensitive(labelt,0);
		gtk_widget_set_sensitive(button1,0);
		gtk_widget_set_sensitive(button2,0);
		gtk_widget_set_sensitive(button3,0);
        }

	button=gtk_check_button_new_with_label(_("Show hidden files"));
	gtk_box_pack_start (GTK_BOX (vbox),button, TRUE, TRUE, 0);
	gtk_signal_connect_object (GTK_OBJECT (button), "toggled",
		hiddenfile_callback, GTK_OBJECT(button));
	if(options.ShowHiddenFiles)gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (button), TRUE);

	button=gtk_check_button_new_with_label(_("Sort directories first"));
	gtk_box_pack_start (GTK_BOX (vbox),button, TRUE, TRUE, 0);
	gtk_signal_connect_object (GTK_OBJECT (button), "toggled",
		sortdirectories_callback, GTK_OBJECT(button));
	if(options.SortDirectoriesFirst)gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (button), TRUE);
	
	hbox = gtk_hbox_new (FALSE, 0);
	gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
	txttimeout = gtk_entry_new();
	gtk_widget_set_usize (txttimeout, 15, 0);
	buffer=g_string_new("");
	g_string_sprintf(buffer,"%d",options.Timeout);
	gtk_entry_set_text(GTK_ENTRY(txttimeout),buffer->str);
	g_string_free(buffer,TRUE);
	gtk_box_pack_start (GTK_BOX (hbox), txttimeout, TRUE, TRUE, 0);
	label = gtk_label_new (_("Timeout for reading from socket (in seconds)"));
	gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);


	/*boutons ok/cancel*/
	bbox = gtk_hbutton_box_new ();
	gtk_box_pack_start (GTK_BOX (vbox), bbox, FALSE, FALSE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (bbox), 10);
	gtk_button_box_set_layout (GTK_BUTTON_BOX (bbox), 
		     GTK_BUTTONBOX_SPREAD );
	button = gtk_button_new_with_label(_("OK"));
	gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
		change_options, GTK_OBJECT(window));
	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
	gtk_container_add (GTK_CONTAINER (bbox), button);
	gtk_widget_grab_default (button);
	button = gtk_button_new_with_label(_("Cancel"));
	gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
		gtk_widget_destroy, GTK_OBJECT(window));
	gtk_container_add (GTK_CONTAINER (bbox), button);

	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
	gtk_widget_show_all(vbox);
	gtk_widget_show(window);
	gtk_main();
}

static GString *getline(FILE *fichier)
{
	GString *temp;
	gchar buf[128];
	
	temp=g_string_new("");
	do
	{
		fgets(buf,sizeof(buf),fichier);
		g_string_append(temp,buf);
		if(feof(fichier))return temp;
	}while(strstr(buf,"\n")==NULL);
	return temp;
}

void read_options()
{
	FILE *fichier;
	GString *buf,*buf2;
	gchar *c;
	
	
	buf=g_string_new("");
	g_string_sprintf(buf,"%s/."PACKAGE"/options",getenv("HOME"));
	if(NULL==(fichier=fopen(buf->str,"rt")))
	{
		options.ShowToolbar=1;
		options.ShowHiddenFiles=0;
		options.SortDirectoriesFirst=1;
		options.Timeout=120;
		options.ToolBarType=0;
		g_string_free(buf,TRUE);
		return;
	}
	g_string_free(buf,TRUE);
	while(!feof(fichier))
	{
		buf=getline(fichier);
                if(feof(fichier))
		{
/*			printf("%d %d %d %d %d\n",options.ShowToolbar,options.ToolBarType,options.ShowHiddenFiles,options.SortDirectoriesFirst,options.Timeout);
*/			fclose(fichier);
			g_string_free(buf,TRUE);
			return;
		}
		if(buf->str[0]!='#')
		{
			buf2=g_string_new("");
			c=buf->str;
			while(*c!=':' && *c!='\n')
			{
				g_string_append_c(buf2,*c);
				c++;
			}
			c++;
			/*printf("%s   %d\n",buf2->str,atoi(c));
			*/if(0==strcmp(buf2->str,"toolbar"))options.ShowToolbar=atoi(c);
			if(0==strcmp(buf2->str,"toolbartype"))options.ToolBarType=atoi(c);
			if(0==strcmp(buf2->str,"hiddenfiles"))options.ShowHiddenFiles=atoi(c);
			if(0==strcmp(buf2->str,"sortdirectories"))options.SortDirectoriesFirst=atoi(c);
			if(0==strcmp(buf2->str,"timeout"))options.Timeout=atoi(c);
			g_string_free(buf2,TRUE);
		}
		g_string_free(buf,TRUE);
	}
	fclose(fichier);
/*	printf("%d %d %d %d %d\n",options.ShowToolbar,options.ToolBarType,options.ShowHiddenFiles,options.SortDirectoriesFirst,options.Timeout);
*/
}
